<?php
if(!defined("__DATE_FUNCTIONS_PHP")){
	define("__DATE_FUNCTIONS_PHP", 1);

/*** return the name of a month given it's number (1 == January, 2 == February, ... 12 == December) ***/
function monthname($monthnum){
	$returnval = "error";
	$monthnum = ($monthnum - 1) % 12 + 1;
	switch($monthnum){
		case 1: $returnval = "January"; break;
		case 2: $returnval = "February"; break;
		case 3: $returnval = "March"; break;
		case 4: $returnval = "April"; break;
		case 5: $returnval = "May"; break;
		case 6: $returnval = "June"; break;
		case 7: $returnval = "July"; break;
		case 8: $returnval = "August"; break;
		case 9: $returnval = "September"; break;
		case 10: $returnval = "October"; break;
		case 11: $returnval = "November"; break;
		case 12: $returnval = "December"; break;
	}
	return $returnval;
}

/********
This function will take a date and add $offset days to it.
It alters the actual vaues passed.
********/
function increment_date(&$year, &$month, &$day, $offset){

        $increment = sgn($offset);
        while($offset != 0){
                $offset -= $increment;
                $day += $increment;

                if($day < 1){
                        $month --;
                        if($month < 1){
                                $month = 12;
                                $year --;
                        }

                        $day = daysinmonth($month, $year); 
                }

                if($day > daysinmonth($month, $year)){
                        $day = 1;
                        $month ++;
                        if($month > 12){
                                $month = 1;
                                $year ++;
                        }
                }
        }
        if($day < 10 && strlen($day) < 2) $day = "0".$day;
        if($month < 10 && strlen($month) < 2) $month = "0".$month;
        if($year < 10 && strlen($year) < 2) $year = "0".$year;
}

/**
returns the number of days in the specified month
**/
function daysinmonth($month, $year){ 
        if($month == 2){
                if(isaleapyear($year)){
                        $returnval = 29;
                }else{
                        $returnval = 28;
		}
        }else{  
                switch($month){
                        case 4: case 6: case 9: case 11:
                                $returnval = 30;
                                break;  
                        default:
                                $returnval = 31;
                }
        }

        return $returnval;
}

/***
returns 1 if the given year is a leap-year, otherwise returns 0.
***/
function isaleapyear($year){
	$returnval = 0;
        if(!($year % 400)) $returnval = 1;
        else if(!($year % 100)) $returnval = 0;
        else if(!($year % 4)) $returnval = 1;
        return $returnval;
}

/***
returns the sign of the number $number
***/
function sgn($number){
        if($number < 0) $returnval = -1;
        else $returnval = ($number > 0);
        return $returnval;
}

/*********
This is an algorithm called "Zeller's Congruence", This version is based upon 
a version I found on the internet which was written in Pascal by Earl Dunovant  
It calculates the weekday of any date.
*********/
function day_of_week($Year, $Month, $Day){
        if($Year < 0 || $Month < 1 || $Month > 12 || $Day < 1 || $Day > 31){
                $returnval = -1;
        }else{
                /* First we separate out the year and the century figures */
                $Century = (int)($Year / 100);
                $Year %= 100;

		/** now make the months go from 3-14 rather than 1-12 **/
                if($Month < 3){
                        $Month += 12;
                        if($Year > 0)
                                $Year --;
                        else{ /* The year before 2000 is 1999, not "20-1" */
                                $Year = 99;
                                $Century --;
                        }
                }

                /****** Here's the core of Zeller's algorithm ********/
                $Holder = $Day;                                 /* Start with the day of month */
                $Holder += (int)((($Month + 1) * 26) / 10);     /* Calculate the incremement */
                $Holder += $Year;                               /* Add in the year */
                $Holder += (int)($Year / 4);                    /* Correct for leap years */
                $Holder += (int)($Century / 4);                 /* Correct for century years */
                $Holder = $Holder - $Century - $Century;        /* DON'T KNOW WHY HE DID THIS! */

                $Holder %= 7;
                if($Holder < 0){
                        $holder += 7;
                }

                /**** "wrap" Saturday around to be the last day ****/
                if($Holder == 0) $Holder = 7;
                $Holder --;
                $returnval = $Holder;
        }
        return $returnval;
}

/***
Draws a quick form for people to select a date.  This is written as a
separate function not for portability, but because dates are asked for
throughout the program.  This cuts down on code size.
***/
function draw_date_selection($yearvarname, $monthvarname, $dayvarname, $offset){
		/*** now draw fields for the date ***/
		$today = getdate();
		$this_year = $today["year"];
		$this_month = $today["mon"];
		$this_day = $today["mday"];

		increment_date($this_year, $this_month, $this_day, $offset);
		/** select a day of the month **/
		echo "<select name=".$dayvarname.">\n";
		for($n = 1; $n <= 31; $n++){
			echo "<option";
			if($n == $this_day) echo " selected";
			echo " value=".$n.">".$n."</option>\n";
		}
		echo "</select>\n";

		/** select a month **/
		echo "<select name=".$monthvarname.">\n";
		for($n = 1; $n <= 12; $n++){
			echo "<option";
			if($n == $this_month) echo " selected";
			echo " value=".$n.">".monthname($n)."</option>\n";
		}
		echo "</select>\n";

		/** select a year **/
		echo "<input type=text name=".$yearvarname." size=4 value=\"".$this_year."\">\n";
}

} /* #ifndef __DATE_FUNCTIONS_PHP */
